import Action from './Action';
import * as Redux from 'redux';

interface ReducerItem<State> {
  reducer: Redux.Reducer<State>;
}

class ReducerContext {
  private static instance: ReducerContext = new ReducerContext();
  private reducerMap: Map<string, ReducerItem<any>[]>;

  private constructor() {
    this.reducerMap = new Map();
  }

  public static getInstance(): ReducerContext {
    return ReducerContext.instance;
  }

  public static putReducers(handlers: Redux.ReducersMapObject): void {
    for(let type in handlers) {
      if (handlers.hasOwnProperty(type)) {
        ReducerContext.getInstance().putReducer(type, handlers[type]);
      }
    }
  }

  public static reducer<State, A extends Action>(state: State, action: A): State {
    const actionType = action.type;
    if (ReducerContext.getInstance().reducerMap.has(actionType)) {
      let reducerItems = ReducerContext.getInstance().reducerMap.get(actionType);
      for (let reducerItem of reducerItems) {
        let reducer = reducerItem.reducer;
        state = reducer(state, action);
      }
    } else {
      console.log(`Reducer for [${actionType}] is not exist in the context.`);
    }
    return state;
  }

  private putReducer<State>(type: string, reducer: Redux.Reducer<State>): void {
    let reducerItems: ReducerItem<any>[];
    if (!this.reducerMap.has(type)) {
      reducerItems = new Array();
      this.reducerMap.set(type, reducerItems);
    } else {
      reducerItems = this.reducerMap.get(type);
      console.log(`Count of reducer[${type}] is ${reducerItems.length + 1}`);
    }
    let reducerItem: ReducerItem<State> = {
      reducer: reducer,
    };

    reducerItems.push(reducerItem);
  }

}

export default ReducerContext;
